#include <windows.h>
#include <stdio.h>

#ifndef DM_INTERLACED
	#define DM_INTERLACED           0x00000002
#endif

#ifndef WPF_ASYNCWINDOWPLACEMENT
	#define WPF_ASYNCWINDOWPLACEMENT    0x0004
#endif


static HMONITOR	g_hMon = NULL;
static RECT		g_RectMon;
static RECT		g_RectPrimaryMon;
static int		g_nMonitorNumber = 0;
static char		g_szDevice [ 128 ];
static int		g_NbWndProcess = 0;
static int		g_NbVisibleWndProcess = 0;
static HWND		g_hWndProcess [ 32 ];

BOOL CALLBACK MonitorEnumProc( HMONITOR hMonitor, HDC hdcMonitor, LPRECT lprcMonitor, LPARAM dwData )
{
	MONITORINFOEX	mi;

	if ( lprcMonitor -> left == 0 && lprcMonitor -> top == 0 )
	{
		// Save primary monitor rectangle
		g_RectPrimaryMon = * lprcMonitor;
	}

	// Test if it is the requested monitor
	if ( g_nMonitorNumber == 0 )
	{
		// Primary monitor: use simple method
		if ( lprcMonitor -> left == 0 && lprcMonitor -> top == 0 )
		{
			g_hMon = hMonitor;
			g_RectMon = * lprcMonitor;
		}
	}
	else
	{
		// Other monitor: test device name
		mi.cbSize = sizeof ( mi );
		GetMonitorInfo ( hMonitor, & mi );
		
		if ( strcmp ( mi.szDevice, g_szDevice ) == 0 )
		{
			g_hMon = hMonitor;
			g_RectMon = * lprcMonitor;
		}
	}

	return ( g_hMon == NULL );
}

BOOL CALLBACK EnumWindowsProc ( HWND hWnd, LPARAM lParam )
{
	DWORD					dwProcessID;
	PROCESS_INFORMATION *	ppi = (PROCESS_INFORMATION *) lParam;
	
	GetWindowThreadProcessId ( hWnd, & dwProcessID );

	if ( dwProcessID == ppi -> dwProcessId )
	{
		g_hWndProcess [ g_NbWndProcess ++ ] = hWnd;
		if ( IsWindowVisible ( hWnd ) )
			g_NbVisibleWndProcess ++;
	}
	
	return TRUE;
}

int WINAPI WinMain ( HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nCmdShow )
{
	int					i, j;
	int					nLoop, nRet, nMon;
	int					nProcessInitTime = 0;
	int					nSleepAndResetAgain = 0;
	int					nWidth, nHeight;
	int					nPixelBits;
	int					nFrequency;
	BOOL				bInterlaced = FALSE;
	BOOL				bWaitAndRestore = FALSE;
	BOOL				bRestoreFromRegistry = FALSE;
	BOOL				bTrackProcessWindow = FALSE;
	BOOL				bMoveInvisibleWindows = FALSE;
	BOOL				bSaveLog = FALSE;
	BOOL				bRet;
	BOOL				bSettingsRetrieved = FALSE;
	BOOL				bSettingsChanged = FALSE;
	BOOL				bKeepActualSettings = FALSE;
	BOOL				bNotAlreadyOnDesktop = FALSE;
	BOOL				bPostMaximize;
	LPSTR				lpRun;
	LPSTR				lpStr, lpStr2;
	char				szBuf [ 512 ];
	char				szBuf2 [ 256 ];
	char				szFileName [ MAX_PATH ];
	char				szLogName [ MAX_PATH ];
	char				szWindowTitleFilter [ 256 ] = "";
	char				szRegRootKey [ 512 ] = "";
	char				szRegValueList [ 128 ] = "";
	HKEY				nKeyCode = HKEY_LOCAL_MACHINE;
	int					NbRegValues = 0;
	char				szRegValueName [ 64 ] [ 128 ];
	DWORD				szRegValueType [ 64 ];
	BYTE *				szRegValueData [ 64 ];
	DWORD				szRegValueSize [ 64 ];
	DWORD				dw;
	unsigned __int64	qw;
	DEVMODE				dm, dmOrg;
	STARTUPINFO			si;
    PROCESS_INFORMATION pi;
	DISPLAY_DEVICE		dd;
	WINDOWPLACEMENT		wp;
	HKEY				hKey = NULL;
	HANDLE				hLogFile = INVALID_HANDLE_VALUE;
	const int			dmSetFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT | DM_DISPLAYFLAGS | DM_DISPLAYFREQUENCY;
	HINSTANCE			hUser;
	
	typedef BOOL ( __stdcall * EnumDisplayProc ) ( LPCSTR, DWORD, PDISPLAY_DEVICEA, DWORD );
	EnumDisplayProc		pEnumDisplayDevices = NULL;

	memset ( & dm, 0, sizeof ( dm ) );
	dm.dmSize = sizeof ( dm );
	
	memset ( & dmOrg, 0, sizeof ( dmOrg ) );
	dmOrg.dmSize = sizeof ( dmOrg );

	// Retrieve module path and name
	GetModuleFileName ( NULL, szFileName, sizeof ( szFileName ) );
	strcpy ( szLogName, szFileName );

	// Build ini file name
	lpStr = strrchr ( szFileName, (int) '.' );
	strcpy ( lpStr + 1, "ini" );

	// Build ini file name
	lpStr = strrchr ( szLogName, (int) '.' );
	strcpy ( lpStr + 1, "log" );

	// Retrieve default startup info
	memset ( & si, 0, sizeof ( si ) );
	si.cb = sizeof ( si );
	GetStartupInfo ( & si );

	// Retrieve EnumDisplayDevices procedure (not in VC6 user32.lib)
	hUser = GetModuleHandle ( "USER32.DLL" );
	pEnumDisplayDevices = (EnumDisplayProc) GetProcAddress ( hUser, "EnumDisplayDevicesA" );

	lpRun = strchr ( lpCmdLine, (int) ' ' );
	lpStr = strchr ( lpCmdLine, (int) '\t' );
	if ( ( lpRun == NULL && lpStr ) || ( lpRun && lpStr && lpStr < lpRun ) )
		lpRun = lpStr;
	
	if ( lpRun )
	{
		lpRun [ 0 ] = '\0';
		do
		{
			lpRun ++;
		} while ( lpRun [ 0 ] == ' ' || lpRun [ 0 ] == '\t' );

		// Retrieve screen profile
		GetPrivateProfileString	( lpCmdLine, "MonitorDevice", "0", szBuf, sizeof ( szBuf ), szFileName );
		g_nMonitorNumber = atoi ( szBuf );

		GetPrivateProfileString	( lpCmdLine, "PixelWidth", "-1", szBuf, sizeof ( szBuf ), szFileName );
		nWidth = atoi ( szBuf );

		GetPrivateProfileString	( lpCmdLine, "PixelHeight", "-1", szBuf, sizeof ( szBuf ), szFileName );
		nHeight = atoi ( szBuf );

		GetPrivateProfileString	( lpCmdLine, "PixelBits", "32", szBuf, sizeof ( szBuf ), szFileName );
		nPixelBits = atoi ( szBuf );

		GetPrivateProfileString	( lpCmdLine, "Frequency", "60", szBuf, sizeof ( szBuf ), szFileName );
		nFrequency = atoi ( szBuf );

		GetPrivateProfileString	( lpCmdLine, "Interlaced", "No", szBuf, sizeof ( szBuf ), szFileName );
		if ( stricmp ( szBuf, "Yes" ) == 0 )
			bInterlaced = TRUE;
		else
			bInterlaced = FALSE;

		GetPrivateProfileString	( lpCmdLine, "RestoreWhenExit", "No", szBuf, sizeof ( szBuf ), szFileName );
		if ( stricmp ( szBuf, "Yes" ) == 0 )
			bWaitAndRestore = TRUE;
		else
			bWaitAndRestore = FALSE;

		GetPrivateProfileString	( lpCmdLine, "RestoreFromRegistry", "Yes", szBuf, sizeof ( szBuf ), szFileName );
		if ( stricmp ( szBuf, "Yes" ) == 0 )
			bRestoreFromRegistry = TRUE;
		else
			bRestoreFromRegistry = FALSE;

		GetPrivateProfileString	( lpCmdLine, "TrackWindow", "No", szBuf, sizeof ( szBuf ), szFileName );
		if ( stricmp ( szBuf, "Yes" ) == 0 )
			bTrackProcessWindow = TRUE;
		else
			bTrackProcessWindow = FALSE;

		GetPrivateProfileString	( lpCmdLine, "MoveInvisibleWindows", "No", szBuf, sizeof ( szBuf ), szFileName );
		if ( stricmp ( szBuf, "Yes" ) == 0 )
			bMoveInvisibleWindows = TRUE;
		else
			bMoveInvisibleWindows = FALSE;
		
		GetPrivateProfileString	( lpCmdLine, "ProcessInitTime", "1000", szBuf, sizeof ( szBuf ), szFileName );
		nProcessInitTime = atoi ( szBuf );

		GetPrivateProfileString	( lpCmdLine, "SleepAndResetAgain", "0", szBuf, sizeof ( szBuf ), szFileName );
		nSleepAndResetAgain = atoi ( szBuf );

		GetPrivateProfileString	( lpCmdLine, "WindowTitleFilter", "", szWindowTitleFilter, sizeof ( szWindowTitleFilter ), szFileName );

		GetPrivateProfileString	( lpCmdLine, "RegistryRootKey", "", szRegRootKey, sizeof ( szRegRootKey ), szFileName );
		GetPrivateProfileString	( lpCmdLine, "RegistryValueList", "", szRegValueList, sizeof ( szRegValueList ), szFileName );

		GetPrivateProfileString	( lpCmdLine, "SaveLog", "No", szBuf, sizeof ( szBuf ), szFileName );
		if ( stricmp ( szBuf, "Yes" ) == 0 )
		{
			hLogFile = CreateFile ( szLogName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );
			if ( hLogFile != INVALID_HANDLE_VALUE )
				bSaveLog = TRUE;
		}
		else
			bSaveLog = FALSE;
		
		if ( szRegRootKey [ 0 ] != '\0' && szRegValueList [ 0 ] != '\0' )
		{
			// Modify registry before anything else
			lpStr = szRegRootKey;
			while ( lpStr [ 0 ] == '\\' )
				lpStr ++;

			lpStr2 = strchr ( lpStr, (int) '\\' );
			lpStr2 [ 0 ] = '\0';
			if ( stricmp ( lpStr, "HKEY_CURRENT_USER" ) == 0 )
				nKeyCode = HKEY_CURRENT_USER;
			else if ( stricmp ( lpStr, "HKEY_CLASSES_ROOT" ) == 0 )
				nKeyCode = HKEY_CLASSES_ROOT;
			else if ( stricmp ( lpStr, "HKEY_CURRENT_CONFIG" ) == 0 )
				nKeyCode = HKEY_CURRENT_CONFIG;
			else
				nKeyCode = HKEY_LOCAL_MACHINE;

			lpStr2 [ 0 ] = '\\';
			lpStr2 ++;

			if ( RegOpenKeyEx ( nKeyCode, lpStr2, 0, KEY_ALL_ACCESS, & hKey ) != ERROR_SUCCESS )
			{
				hKey = NULL;

				if ( bSaveLog )
				{
					sprintf ( szBuf, "Cannot open registry key: %s\\%s\r\n", ( nKeyCode == HKEY_CURRENT_USER ? "HKEY_CURRENT_USER" : nKeyCode == HKEY_CLASSES_ROOT ? "HKEY_CLASSES_ROOT" : nKeyCode == HKEY_CURRENT_CONFIG ? "HKEY_CURRENT_CONFIG" : "HKEY_LOCAL_MACHINE" ), lpStr2 );
					WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
				}
			}
			else
			{
				if ( bSaveLog )
				{
					sprintf ( szBuf, "Registry key %s successfully opened.\r\n", lpStr2 );
					WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
				}
			}
		}

		if ( nWidth < 0 || nHeight < 0 || nPixelBits < 0 || nFrequency < 0 )
		{
			bKeepActualSettings = TRUE;
		}
		else
		{
			dm.dmFields = dmSetFields;
			dm.dmPelsWidth = nWidth;
			dm.dmPelsHeight = nHeight;
			dm.dmBitsPerPel = nPixelBits;
			dm.dmDisplayFrequency = nFrequency;
			dm.dmDisplayFlags = bInterlaced ? DM_INTERLACED : 0;
		}

		bRet = FALSE;
		if ( pEnumDisplayDevices )
		{
			dd.cb = sizeof ( dd );
			for ( nMon = 0, nLoop = 0 ; pEnumDisplayDevices ( NULL, nLoop, & dd, 0 ) ; nLoop ++ )
			{
				if ( ( dd.StateFlags & ( DISPLAY_DEVICE_MIRRORING_DRIVER | DISPLAY_DEVICE_ATTACHED_TO_DESKTOP ) ) != DISPLAY_DEVICE_MIRRORING_DRIVER )
				{
					if ( bSaveLog )
					{
						sprintf ( szBuf, "Monitor %d found: %s (%s) State = 0x%08X\r\n", nMon, dd.DeviceName, dd.DeviceString, dd.StateFlags );
						WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
					}

					if ( g_nMonitorNumber == nMon )
					{
						bRet = TRUE;
						strcpy ( g_szDevice, dd.DeviceName );

						if ( ( dd.StateFlags & DISPLAY_DEVICE_ATTACHED_TO_DESKTOP ) == 0 )
							bNotAlreadyOnDesktop = TRUE;

						if ( ! bSaveLog )
						{
							// No need to continue
							break;
						}
					}
					nMon ++;
				}
				else
				{
					if ( bSaveLog )
					{
						sprintf ( szBuf, "Pseudo-device (ignored): %s (%s) State = 0x%08X\r\n", dd.DeviceName, dd.DeviceString, dd.StateFlags );
						WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
					}
				}
			}
		}

		if ( bRet )
		{
			// Monitor found
			if ( hKey )
			{
				// Retrieve current values from registry and set new ones
				lpStr = (char *) malloc ( 8000 );
				GetPrivateProfileString	( szRegValueList, NULL, "", lpStr, 8000, szFileName );
				
				NbRegValues = 0;
				
				lpStr2 = lpStr;
				while ( lpStr2 [ 0 ] != '\0' )
				{
					strcpy ( szRegValueName [ NbRegValues ], lpStr2 );
					lpStr2 = strchr ( lpStr2, 0 ) + 1;

					if ( NbRegValues < sizeof ( szRegValueSize ) / sizeof ( szRegValueSize [ 0 ] ) )
						NbRegValues ++;
				}

				free ( lpStr );

				for ( i = 0 ; i < NbRegValues ; i ++ )
				{
					// Retrieve actual value
					szRegValueSize [ i ] = 0;
					szRegValueData [ i ] = NULL;
					if ( ERROR_SUCCESS == RegQueryValueEx ( hKey, szRegValueName [ i ], NULL, & szRegValueType [ i ], NULL, & szRegValueSize [ i ] ) )
					{
						szRegValueData [ i ] = (BYTE *) malloc ( szRegValueSize [ i ] + 1 );
						RegQueryValueEx ( hKey, szRegValueName [ i ], NULL, NULL, szRegValueData [ i ], & szRegValueSize [ i ] );
						if ( bSaveLog )
						{
							sprintf ( szBuf, "Registry value %s saved and updated.\r\n", szRegValueName [ i ] );
							WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
						}
					}
					else
					{
						szRegValueType [ i ] = REG_NONE;
						
						if ( bSaveLog )
						{
							sprintf ( szBuf, "Registry value %s temporarily created.\r\n", szRegValueName [ i ] );
							WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
						}
					}

					// Retrieve requested value in .ini file
					GetPrivateProfileString	( szRegValueList, szRegValueName [ i ], "", szBuf, sizeof ( szBuf ), szFileName );

					// Set value
					switch ( szRegValueType [ i ] )
					{
						case REG_DWORD:
							 dw = atol ( szBuf );
							 RegSetValueEx ( hKey, szRegValueName [ i ], 0, REG_DWORD, (unsigned char *) & dw, sizeof ( dw ) );
							 break;

						case REG_QWORD:
							 qw = _atoi64 ( szBuf );
							 RegSetValueEx ( hKey, szRegValueName [ i ], 0, REG_QWORD, (unsigned char *) & qw, sizeof ( qw ) );
							 break;

						case REG_SZ:
						case REG_EXPAND_SZ:
							 RegSetValueEx ( hKey, szRegValueName [ i ], 0, szRegValueType [ i ], (unsigned char *) szBuf, strlen ( szBuf ) );
							 break;

						case REG_BINARY:
							 for ( j = 0 ; szBuf [ j * 2 ] != '\0' && szBuf [ j * 2 + 1 ] != '\0' ; j ++ )
							 {
								szBuf2 [ j ] = ( isxdigit ( szBuf [ j * 2 ] ) ? ( isdigit ( szBuf [ j * 2 ] ) ? ( szBuf [ j * 2 ] - '0' ) : ( tolower ( szBuf [ j * 2 ] ) - 'a' + 10 ) ) : 0 ) * 16 + ( isxdigit ( szBuf [ j * 2 + 1 ] ) ? ( isdigit ( szBuf [ j * 2 + 1 ] ) ? ( szBuf [ j * 2 + 1 ] - '0' ) : ( tolower ( szBuf [ j * 2 + 1 ] ) - 'a' + 10 ) ) : 0 );
							 }
							 RegSetValueEx ( hKey, szRegValueName [ i ], 0, szRegValueType [ i ], (unsigned char *) szBuf2, j );
							 break;

						case REG_NONE:
							 RegSetValueEx ( hKey, szRegValueName [ i ], 0, REG_SZ, (unsigned char *) szBuf, strlen ( szBuf ) );
							 break;

						default:
							 // unhandled type: do nothing, so avoid restoring initial value.
							 szRegValueSize [ i ] = 0;
							 free ( szRegValueData [ i ] );
							 szRegValueData [ i ] = NULL;
							 break;
					}
				}

				RegFlushKey ( hKey );
				Sleep (0);

				// Registry updated: init display from registry
				ChangeDisplaySettingsEx ( g_szDevice, NULL, NULL, 0, NULL );
				if ( nSleepAndResetAgain > 0 )
				{
					Sleep (nSleepAndResetAgain);
					ChangeDisplaySettingsEx ( g_szDevice, NULL, NULL, 0, NULL );
				}
				Sleep (0);
			}

			if ( ! bKeepActualSettings )
			{
				// Retrieve actual settings
				if ( EnumDisplaySettings ( g_szDevice, ENUM_CURRENT_SETTINGS, & dmOrg ) )
				{
					bSettingsRetrieved = TRUE;

					if ( bSaveLog )
					{
						sprintf ( szBuf, "\r\nCurrent settings for monitor %d: %s\r\nSize = (%d,%d) at %d Hertz, %d bits per pixel\r\n", g_nMonitorNumber, g_szDevice, dmOrg.dmPelsWidth, dmOrg.dmPelsHeight, dmOrg.dmDisplayFrequency, dmOrg.dmBitsPerPel );
						WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
					}
				}
				else
				{
					if ( bSaveLog )
					{
						sprintf ( szBuf, "\r\nFail to retrieve current settings for monitor %d: %s\r\n", g_nMonitorNumber, g_szDevice );
						WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
					}
				}

				if ( bNotAlreadyOnDesktop )
				{
					// Attach monitor to the desktop at the right of primary monitor
					dm.dmPosition.x = g_RectPrimaryMon.right;
					dm.dmPosition.y = g_RectPrimaryMon.top;

					dm.dmFields |= DM_POSITION;
				}

				// Change settings
				nRet = ChangeDisplaySettingsEx ( g_szDevice, & dm, NULL, 0, NULL );
				Sleep (0);

				switch ( nRet )
				{
					case DISP_CHANGE_SUCCESSFUL:
						 // Settings changed
						 bSettingsChanged = TRUE;
						 if ( bSaveLog )
						 {
							sprintf ( szBuf, "New settings successfully set.\r\n" );
							WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
						 }
						 break;

					case DISP_CHANGE_RESTART:
						 // "Must restart"... not acceptable: cancel these settings
						 if ( bRestoreFromRegistry || ! bSettingsRetrieved )
						 {
							ChangeDisplaySettingsEx ( g_szDevice, NULL, NULL, 0, NULL );
						 }
						 else
						 {
							dmOrg.dmFields = dmSetFields;
							ChangeDisplaySettingsEx ( g_szDevice, & dmOrg, NULL, 0, NULL );
						 }
						 Sleep (0);
						 if ( bSaveLog )
						 {
							sprintf ( szBuf, "New settings need computer restart to be applied. Changes canceled.\r\n" );
							WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
						 }
						 break;

					case DISP_CHANGE_NOTUPDATED:
					case DISP_CHANGE_FAILED:
					case DISP_CHANGE_BADPARAM:
					case DISP_CHANGE_BADMODE:
					case DISP_CHANGE_BADFLAGS:
					default:
						 // Error
						 if ( bSaveLog )
						 {
							sprintf ( szBuf, "ERROR in settings. ChangeDisplaySettingsEx returned %d.\r\n(-1: Failed, -2: Bad Mode, -3: Not updated, -4: Bad flags, -5: Bad params)\r\n\r\nAvailable display settings:\r\n", nRet );
							WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );

							// Enum all available display settings
							for ( nLoop = 0 ; EnumDisplaySettings ( g_szDevice, nLoop, & dmOrg ) ; nLoop ++ )
							{
								sprintf ( szBuf, "Size = (%d,%d) at %d Hertz, %d bits per pixel\r\n", dmOrg.dmPelsWidth, dmOrg.dmPelsHeight, dmOrg.dmDisplayFrequency, dmOrg.dmBitsPerPel );
								WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
							}
							sprintf ( szBuf, "--- end of list ---\r\n\r\n" );
							WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
						 }
						 break;
				}
			}
		}
	}

	EnumDisplayMonitors ( NULL, NULL, MonitorEnumProc, NULL );
	
	if ( g_hMon )
	{
		// Use monitor rectangle
		si.dwFlags |= STARTF_USEPOSITION | STARTF_USESIZE;
		si.dwX = g_RectMon.left;
		si.dwY = g_RectMon.top;
		si.dwXSize = g_RectMon.right - g_RectMon.left;
		si.dwYSize = g_RectMon.bottom - g_RectMon.top;

		if ( bSaveLog )
		{
			sprintf ( szBuf, "Monitor found at position (%d,%d) on virtual screen.\r\n", g_RectMon.left, g_RectMon.top );
			WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
		}
	}
	else
	{
		if ( bSaveLog )
		{
			sprintf ( szBuf, "ERROR: Monitor %d not found. Will use default display without changes.\r\n", g_nMonitorNumber );
			WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
		}
	}

	if ( lpRun && lpRun [ 0 ] && CreateProcess ( NULL, lpRun, NULL, NULL, FALSE, NORMAL_PRIORITY_CLASS, NULL, NULL, & si, & pi ) )
	{
		if ( g_hMon && bTrackProcessWindow )
		{
			Sleep ( nProcessInitTime );
			
			for ( nLoop = 0 ; g_NbVisibleWndProcess == 0 && nLoop < 20 ; nLoop ++ )
			{
				// Retrieve process main window
				g_NbWndProcess = 0;
				g_NbVisibleWndProcess = 0;
				EnumWindows ( EnumWindowsProc, (LPARAM) & pi );

				if ( g_NbVisibleWndProcess )
				{
					// Process main window found: Check its position and put it on defined monitor if necessary
					for ( i = 0 ; i < g_NbWndProcess ; i ++ )
					{
						GetWindowText ( g_hWndProcess [ i ], szBuf2, sizeof ( szBuf2 ) );

						// Take into account only visible windows which match windows title filter
						if ( ( IsWindowVisible ( g_hWndProcess [ i ] ) || bMoveInvisibleWindows ) && strstr ( szBuf2, szWindowTitleFilter ) != NULL )
						{
							if ( g_hMon != MonitorFromWindow ( g_hWndProcess [ i ], MONITOR_DEFAULTTONULL ) )
							{
								// Not on the requested monitor: move it (asynchroneously because in another process)
								if ( bSaveLog )
								{
									sprintf ( szBuf, "Main application window (%s) found on another monitor: move it to requested monitor.\r\n", szBuf2 );
									WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
								}

								// Retrieve current placement
								memset ( & wp, 0, sizeof ( wp ) );
								wp.length = sizeof ( wp );
								GetWindowPlacement ( g_hWndProcess [ i ], & wp );

								if ( ! IsWindowVisible ( g_hWndProcess [ i ] ) )
									wp.showCmd = SW_HIDE;

								if ( wp.showCmd == SW_MAXIMIZE )
								{
									// Minimize window during monitor change, and maximize it again after
									wp.showCmd = SW_SHOWMINIMIZED;
									bPostMaximize = TRUE;
								}
								else
								{
									// Change directly window position to set it on right monitor
									bPostMaximize = FALSE;
								}

								// Keep only "restore to maximized" flag (if the window is minimized)
								// Note: if window is minimized, its position will not change until it is restored
								if ( wp.flags & WPF_RESTORETOMAXIMIZED )
									wp.flags = WPF_ASYNCWINDOWPLACEMENT | WPF_RESTORETOMAXIMIZED;
								else
									wp.flags = WPF_ASYNCWINDOWPLACEMENT;

								// Define origin of window in maximized mode
								wp.ptMaxPosition.x = g_RectMon.left;
								wp.ptMaxPosition.y = g_RectMon.top;

								// Define origin of window in normal mode
								wp.rcNormalPosition.left = g_RectMon.left + 50;
								wp.rcNormalPosition.top = g_RectMon.top + 50;
								wp.rcNormalPosition.right = g_RectMon.right - 50;
								wp.rcNormalPosition.bottom = g_RectMon.bottom - 50;

								// Move window
								SetWindowPlacement ( g_hWndProcess [ i ], & wp );

								if ( bPostMaximize )
								{
									// Maximize window on requested monitor
									Sleep ( 100 );
									wp.showCmd = SW_SHOWMAXIMIZED;
									wp.flags = WPF_ASYNCWINDOWPLACEMENT | WPF_RESTORETOMAXIMIZED;
									SetWindowPlacement ( g_hWndProcess [ i ], & wp );
								}
							}
							else
							{
								if ( bSaveLog )
								{
									sprintf ( szBuf, "Main application window (%s) found on requested monitor: nothing to do.\r\n", szBuf2 );
									WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
								}
							}
						}
						else
						{
							if ( bSaveLog )
							{
								if ( IsWindowVisible ( g_hWndProcess [ i ] ) || bMoveInvisibleWindows )
									sprintf ( szBuf, "Window (%s) does not match title filter (%s): do nothing.\r\n", szBuf2, szWindowTitleFilter );
								else
									sprintf ( szBuf, "Invisible window (%s) found: do nothing.\r\n", szBuf2 );
								WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
							}
						}
					}
				}
				else
				{
					// Wait a little time before next loop
					Sleep ( 300 );
				}
			}
		}

		if ( bWaitAndRestore && ( bSettingsChanged || NbRegValues > 0 ) )
		{
			if ( bSaveLog )
			{
				sprintf ( szBuf, "Waiting for process ending before resetting original display configuration.\r\n--- EOF ---\r\n" );
				WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );

				// Close log file now to unlock it, so user can read it while target application is still running
				CloseHandle ( hLogFile );
				hLogFile = INVALID_HANDLE_VALUE;
				bSaveLog = FALSE;
			}

			// Wait for the end of the process
			WaitForSingleObject ( pi.hProcess, INFINITE );

			// Restore initial values and close registry key
			if ( NbRegValues > 0 )
			{
				// Restore registry
				for ( i = 0 ; i < NbRegValues ; i ++ )
				{
					// Restore
					switch ( szRegValueType [ i ] )
					{
						case REG_DWORD:
						case REG_QWORD:
						case REG_SZ:
						case REG_EXPAND_SZ:
						case REG_BINARY:
							 RegSetValueEx ( hKey, szRegValueName [ i ], 0, szRegValueType [ i ], szRegValueData [ i ], szRegValueSize [ i ] );
							 break;

						case REG_NONE:
							 // Delete value
							 RegDeleteValue ( hKey, szRegValueName [ i ] );
							 break;

						default:
							 // Error...
							 break;
					}
					
					if ( szRegValueData [ i ] )
					{
						free ( szRegValueData [ i ] );
						szRegValueData [ i ] = NULL;
					}
				}
				RegFlushKey ( hKey );
				Sleep (0);
			}

			// Restore initial settings (do not test return code: if it fails, there is nothing we can do...)
			if ( bRestoreFromRegistry || ! bSettingsRetrieved || NbRegValues > 0 )
			{
				ChangeDisplaySettingsEx ( g_szDevice, NULL, NULL, 0, NULL );
			}

			if ( ! bRestoreFromRegistry && bSettingsChanged && bSettingsRetrieved )
			{
				if ( NbRegValues > 0 )
				{
					// Wait a little between two calls to ChangeDisplaySettingsEx
					Sleep (500);
				}
				
				dmOrg.dmFields = dmSetFields;
				ChangeDisplaySettingsEx ( g_szDevice, & dmOrg, NULL, 0, NULL );
			}
		}
		
		CloseHandle ( pi.hProcess );
		CloseHandle ( pi.hThread );
	}
	else
	{
		MessageBox ( NULL, "Cannot create new process, please check command line\r\nSyntax: Run2mon <ini_profile> <exefile> [parameters]", "Run2mon", MB_ICONHAND | MB_OK );
	}

	if ( hKey )
	{
		RegCloseKey ( hKey );
		hKey = NULL;
	}

	if ( hLogFile != INVALID_HANDLE_VALUE )
	{
		sprintf ( szBuf, "--- EOF ---\r\n" );
		WriteFile ( hLogFile, szBuf, strlen ( szBuf ), & dw, NULL );
		CloseHandle ( hLogFile );
		hLogFile = INVALID_HANDLE_VALUE;
	}

	return 0;
}
